use tokio::sync::OnceCell;

pub mod auth;
pub mod format;
#[cfg(feature = "htmx")]
pub mod htmx;
pub mod page;
pub mod request;
pub mod result;
pub mod db;

// TODO : API Pagination?
// TODO : CORS?
// TODO : CSRF?


// TODO : Implement SQLITE backend
// Code sharing with macro
// Seperate migrations
// which db by $DB env var "postgres" "sqlite"

// Postgres

pub static PG: OnceCell<sqlx::PgPool> = OnceCell::const_new();
pub static SQ: OnceCell<sqlx::SqlitePool> = OnceCell::const_new();

/// A macro to retrieve or initialize the PostgreSQL connection pool.
///
/// This macro provides a convenient way to access the `PgPool`. If the pool is not already initialized,
/// it creates a new pool using the connection string from the `$DATABASE_URL` environment variable.
///
/// # Example
/// ```ignore
/// use based::get_pg;
///
/// let pool = get_pg!();
/// ```
#[macro_export]
macro_rules! get_pg {
    () => {
        if let Some(client) = $crate::PG.get() {
            client
        } else {
            let client = sqlx::postgres::PgPoolOptions::new()
                .max_connections(5)
                .connect(&std::env::var("DATABASE_URL").unwrap())
                .await
                .unwrap();
            $crate::PG.set(client).unwrap();
            $crate::PG.get().unwrap()
        }
    };
}

#[macro_export]
macro_rules! get_sqlite  {
    () => {
        if let Some(client) = $crate::SQ.get() {
            client
        } else {

            let client = sqlx::sqlite::SqlitePoolOptions::new()
                .max_connections(5)
                .connect(&std::env::var("DATABASE_URL").unwrap())
                .await
                .unwrap();
            $crate::SQ.set(client).unwrap();
            $crate::SQ.get().unwrap()
        }
    };
}

#[macro_export]
macro_rules! get_sqlite_or_create  {
    () => {
        if let Some(client) = $crate::SQ.get() {
            client
        } else {
            let db_url = std::env::var("DATABASE_URL").unwrap();

            if !std::fs::exists(&db_url).unwrap_or(false) {
                std::fs::File::create(&db_url).unwrap();
            }

            let client = sqlx::sqlite::SqlitePoolOptions::new()
                .max_connections(5)
                .connect(&db_url)
                .await
                .unwrap();
            $crate::SQ.set(client).unwrap();
            $crate::SQ.get().unwrap()
        }
    };
}
